/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.base.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.base.entity.Config;
import com.tofly.base.service.ConfigService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 系统配置信息表
 *
 * @author admin
 * @date Thu Oct 10 00:00:00 CST 2019
 */
@RestController
@AllArgsConstructor
@RequestMapping("/config")
@Api(tags="系统参数管理")
public class ConfigController {

    private final  ConfigService configService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param config 系统配置信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getConfigPage(Page page, @ModelAttribute Config config) {
      return ResultRespone.success(configService.page(page, Wrappers.query(config)));
    }


    /**
     * 通过id查询系统配置信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(configService.getById(id));
    }

//    /**
//     * 新增系统配置信息表
//     * @param config 系统配置信息表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增系统配置信息表" )
//    @ApiOperation(value = "新增系统配置信息表")
//    @PostMapping
//    public ResultRespone save(@RequestBody Config config) {
//      return ResultRespone.success(configService.save(config));
//    }

    /**
     * 修改系统配置信息表
     * @param config 系统配置信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改系统配置信息表" )
    @ApiOperation(value = "修改系统配置信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Config config) {
      return ResultRespone.success(configService.updateById(config));
    }

//    /**
//     * 通过id删除系统配置信息表
//     * @param id id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id删除系统配置信息表" )
//    @ApiOperation(value = "通过ID删除系统配置信息表")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//      return ResultRespone.success(configService.removeById(id));
//    }
}
