/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.base.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.base.entity.Menu;
import com.tofly.base.entity.MenuBatch;
import com.tofly.base.entity.UserRole;
import com.tofly.base.mapper.MenuMapper;
import com.tofly.base.service.MenuService;
import com.tofly.base.service.RoleMenuService;
import com.tofly.base.service.UserRoleService;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 菜单表
 *
 * @author admin
 * @date Thu Oct 10 00:00:00 CST 2019
 */
@Service("menuService")
public class MenuServiceImpl extends ServiceImpl<MenuMapper, Menu> implements MenuService {

    @Autowired
    private UserRoleService userRoleService;
    @Autowired
    private RoleMenuService roleMenuService;

    @Override
    public List<Menu> getMenuListByRole(Long roleId) {
        return baseMapper.getMenuListByRole(roleId);
    }

    @Override
    public List<Menu> getMenuListByUser(String platform) {

        return baseMapper.getMenuListByUser(SecurityUtils.getUserId(), platform);
    }

    @Override
    @Transactional
    public String removeMenuByIds(List<Long> list) {
        List<Long> listcopy = new ArrayList<>();
        listcopy.addAll(list);
        //校验菜单，从子级一步步进行校验
        List<Menu> menus = (List<Menu>) this.listByIds(list);
        List<Long> newList = new ArrayList();
        for (Menu menu : menus) {
            //校验父级ID是否存在在ID中
            if (list.contains(menu.getParentId())) {
                newList.add(menu.getParentId());
            }
        }
        list.removeAll(newList);
        StringBuffer sb = new StringBuffer();
        //先清除子级菜单
        for (Long id : list) {
            //查询是否存在子菜单
            List<Menu> menuList = this.list(Wrappers.<Menu>lambdaQuery()
                    .eq(Menu::getParentId, id));
            if (menuList.size() > 0) {

                sb.append("id:").append(id).append(",对应子级菜单还存在").append(menuList.size())
                        .append("条数据;");
            }
        }
        if (StringUtil.isEmpty(sb.toString())) {
            this.removeByIds(list);
            if (newList.size() > 0) {
                return removeMenuByIds(newList);
            } else {
                return null;
            }
        } else {
            return null;
        }
    }

    @Override
    @SneakyThrows
    @Transactional
    public String uploadBatchMenu(MultipartFile file) {
        if (file == null) {
            throw new ToflyDeniedException("上传文件为空");
        }

        InputStream inputStream = file.getInputStream();
        ExcelUtil menuExcel = new ExcelUtil(MenuBatch.class);
        List<MenuBatch> listMenu = menuExcel.importExcel(inputStream);
        List<Menu> listAllMenu = this.list();
        Map<String, Menu> collect = listAllMenu.stream().collect(Collectors.toMap(Menu::getName, m -> m, (k1, k2) -> k1));
        StringBuffer sb = new StringBuffer();
        List<String> collect1 = listMenu.stream().filter(k -> collect.get(k.getName()) != null)
                .map(MenuBatch::getName).collect(Collectors.toList());
        if (collect1.size() > 0) {
            throw new ToflyDeniedException("数据库已经存在菜单名称，不能导入" + JSONArray.toJSONString(collect1));
        }

        listMenu.sort((m1, m2) -> m1.getParentName().compareTo(m2.getParentName()));
        listMenu.stream().forEach(menuBatch -> {
            if (menuBatch.getParentId() == null && !StringUtil.isEmpty(menuBatch.getParentName())) {
                menuBatch.setParentId(collect.get(menuBatch.getParentName()).getId());
            }
            this.save(menuBatch);
            collect.put(menuBatch.getName(), menuBatch);
        });
        return "成功导入" + listMenu.size() + "条数据";
    }

    @Override
    public List<Menu> listMenusByRoleId(Long roleId) {
        return baseMapper.listMenusByRoleId(roleId);
    }

    @Override
    public List<Menu> queryAllMenu() {
        //1 查询菜单表所有数据
        QueryWrapper<Menu> wrapper = new QueryWrapper<>();
        wrapper.eq("STATUS_FLAG",1);
        wrapper.orderByAsc("SORT");
        List<Menu> permissionList = baseMapper.selectList(wrapper);
        //2 把查询所有菜单list集合按照要求进行封装
        List<Menu> resultList = bulidPermission(permissionList);
        return resultList;
    }

    @Override
    public List<Menu> getTreeMenuByUser(Long id) {
        //通过用户id获取角色
        List<UserRole> userRoles = userRoleService.getTreeMenuByUser(id);
        List<Long> roles = userRoles.stream().map(UserRole::getRoleId).collect(Collectors.toList());
        //通过角色来获取菜单id
        List<Long> menusIds = roleMenuService.getMenuIds(roles);
        if (CollectionUtils.isEmpty(menusIds)){
            throw new ToflyDeniedException("");
        }
        Set<Long> menusSetId = new HashSet<>(menusIds);
//        List<Menu> menuList = baseMapper.selectBatchIds(menusSetId);
        List<Menu> menuList = this.list(Wrappers.<Menu>lambdaQuery().in(Menu::getId,menusSetId).orderByAsc(Menu::getSort));
        // 菜单去重
        //2 把查询所有菜单list集合按照要求进行封装
        List<Menu> resultList = bulidPermission(menuList);
        return resultList;
    }

    @Override
    public List<Menu> getTreeMenuByUserNew(Long id) {
        List<Menu> menuList = this.baseMapper.getMenuListByUserId(id);
        Map<Long,Menu> map = new HashMap<>(12);
        List<Menu> rt = new ArrayList<>();
        for (Menu m:menuList) {
            m.setChildrens(new ArrayList<>());
            if (m.getParentId() == null) {
                //设置顶层菜单的level是1
                m.setLevel(1);
                rt.add(m);
                map.put(m.getId(),m);
            }else {
                if(map.containsKey(m.getParentId())){
                    Menu fm = map.get(m.getParentId());
                    int level = fm.getLevel() + 1;
                    m.setLevel(level);
                    fm.getChildrens().add(m);
                    map.put(m.getId(),m);
                }
            }
        }
        return rt;
    }

    @Override
    public Menu getUserPersion() {
        QueryWrapper<Menu> wrapper = new QueryWrapper<>();
        wrapper.eq("NAME","UserManagement");
        wrapper.eq("SYS_ID",13);
        Menu menu = baseMapper.selectOne(wrapper);
        return menu;
    }

    //把返回所有菜单list集合进行封装的方法
    private List<Menu> bulidPermission(List<Menu> permissionList) {
        //创建list集合，用于数据最终封装
        List<Menu> finalNode = new ArrayList<>();
        //把所有菜单list集合遍历，得到顶层菜单 pid=null菜单，设置level是1
        for (Menu menu : permissionList) {
            //得到顶层菜单 pid=null菜单
            if (menu.getParentId() == null) {
                //设置顶层菜单的level是1
                menu.setLevel(1);
                //根据顶层菜单，向里面进行查询子菜单，封装到finalNode里面
                finalNode.add(selectChildren(menu, permissionList));
            }
        }
        return finalNode;
    }

    private Menu selectChildren(Menu menu, List<Menu> permissionList) {
        //1 因为向一层菜单里面放二层菜单，二层里面还要放三层，把对象初始化
        menu.setChildrens(new ArrayList<Menu>());

        //2 遍历所有菜单list集合，进行判断比较，比较id和pid值是否相同
        for (Menu it : permissionList) {
            //判断 id和pid值是否相同
            if (menu.getId().equals(it.getParentId())) {
                //把父菜单的level值+1
                int level = menu.getLevel() + 1;
                it.setLevel(level);
                //如果children为空，进行初始化操作
                if (menu.getChildrens() == null) {
                    menu.setChildrens(new ArrayList<Menu>());
                }
                //把查询出来的子菜单放到父菜单里面
                menu.getChildrens().add(selectChildren(it, permissionList));
            }
        }
        return menu;
    }

}
