/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.entity.Itemtyperato;
import com.tofly.xrtygis.service.ItemtyperatoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 指标类别频率归集信息表
 *
 * @author admin
 * @date Sat Apr 17 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/itemtyperato")
@Api(tags = "指标类别频率归集信息表接口")
public class ItemtyperatoController {

    private final ItemtyperatoService itemtyperatoService;

    /**
     * 分页查询
     *
     * @param page         分页对象
     * @param itemtyperato 指标类别频率归集信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getItemtyperatoPage(Page page, @ModelAttribute Itemtyperato itemtyperato) {
        return ResultRespone.success(itemtyperatoService.page(page, Wrappers.query(itemtyperato)));
    }


    /**
     * 通过id查询指标类别频率归集信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(itemtyperatoService.getById(id));
    }

    /**
     * 新增指标类别频率归集信息表
     *
     * @param itemtyperato 指标类别频率归集信息表
     * @return ResultRespone
     */
    //(title = "新增指标类别频率归集信息表")
    @ApiOperation(value = "新增指标类别频率归集信息表")
    @PostMapping
    public ResultRespone save(@RequestBody Itemtyperato itemtyperato) {
        return ResultRespone.success(itemtyperatoService.save(itemtyperato));
    }

    /**
     * 修改指标类别频率归集信息表
     *
     * @param itemtyperato 指标类别频率归集信息表
     * @return ResultRespone
     */
    //(title = "修改指标类别频率归集信息表")
    @ApiOperation(value = "修改指标类别频率归集信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Itemtyperato itemtyperato) {
        return ResultRespone.success(itemtyperatoService.updateById(itemtyperato));
    }

    /**
     * 通过id删除指标类别频率归集信息表
     *
     * @param id id
     * @return ResultRespone
     */
    //(title = "通过id删除指标类别频率归集信息表")
    @ApiOperation(value = "通过ID删除指标类别频率归集信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(itemtyperatoService.removeById(id));
    }

    /**
     * 通过id批量删除指标类别频率归集信息表
     *
     * @param ids id
     * @return ResultRespone
     */
    //(title = "通过id批量删除指标类别频率归集信息表")
    @ApiOperation(value = "通过ID批量删除指标类别频率归集信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(itemtyperatoService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
