/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.entity.TfLeakAnalyse;
import com.tofly.xrtygis.service.TfLeakAnalyseService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 漏损率分析记录表
 *
 * @author JF
 * @date Fri May 27 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfleakanalyse")
@Api(tags="漏损率分析记录表接口")
public class TfLeakAnalyseController {

    private final  TfLeakAnalyseService tfLeakAnalyseService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfLeakAnalyse 漏损率分析记录表
    * @return
    */
    @ToFlyAppLog(title="分页查询" )
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfLeakAnalysePage(Page page, @ModelAttribute TfLeakAnalyse tfLeakAnalyse) {
      return ResultRespone.success(tfLeakAnalyseService.page(page, Wrappers.query(tfLeakAnalyse)));
    }


    /**
     * 通过id查询漏损率分析记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过ID查询" )
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfLeakAnalyseService.getById(id));
    }

    /**
     * 新增漏损率分析记录表
     * @param tfLeakAnalyse 漏损率分析记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增漏损率分析记录表" )
    @ApiOperation(value = "新增漏损率分析记录表")
    @PostMapping
    public ResultRespone save(@RequestBody TfLeakAnalyse tfLeakAnalyse) {
      return ResultRespone.success(tfLeakAnalyseService.save(tfLeakAnalyse));
    }

    /**
     * 修改漏损率分析记录表
     * @param tfLeakAnalyse 漏损率分析记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改漏损率分析记录表" )
    @ApiOperation(value = "修改漏损率分析记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody TfLeakAnalyse tfLeakAnalyse) {
      return ResultRespone.success(tfLeakAnalyseService.updateById(tfLeakAnalyse));
    }

    /**
     * 通过id删除漏损率分析记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除漏损率分析记录表" )
    @ApiOperation(value = "通过ID删除漏损率分析记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(tfLeakAnalyseService.removeById(id));
    }

    /**
     * 通过id批量删除漏损率分析记录表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除漏损率分析记录表" )
    @ApiOperation(value = "通过ID批量删除漏损率分析记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(tfLeakAnalyseService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
