/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.xrtygis.entity.Views;
import com.tofly.xrtygis.service.ViewsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;

/**
 * 工作书签信息表
 *
 * @author admin
 * @date Wed Mar 10 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/views")
@Api(tags = "工作书签信息表接口")
public class ViewsController {

    private final ViewsService viewsService;

    /**
     * 分页查询
     *
     * @param page  分页对象
     * @param views 工作书签信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getViewsPage(Page page, @ModelAttribute Views views) {
        return ResultRespone.success(viewsService.page(page, Wrappers.<Views>lambdaQuery().orderByDesc(Views::getCreateTime)));
    }

    @GetMapping("/getMe")
    @ApiOperation("获取当前用户创建书签")
    public ResultRespone getMe(Page page) {
        Page page1 = viewsService.page(page, Wrappers.<Views>lambdaQuery().eq(Views::getCreateUser, SecurityUtils.getUserId()).orderByDesc(Views::getCreateTime));
        return ResultRespone.success(page1);
    }


    /**
     * 通过id查询工作书签信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(viewsService.getById(id));
    }

    /**
     * 新增工作书签信息表
     *
     * @param views 工作书签信息表
     * @return ResultRespone
     */
    //(title = "新增工作书签信息表")
    @ApiOperation(value = "新增工作书签信息表")
    @PostMapping
    public ResultRespone save(@RequestBody Views views) {
        views.setCreateTime(new Date());
        return ResultRespone.success(viewsService.save(views));
    }

    /**
     * 修改工作书签信息表
     *
     * @param views 工作书签信息表
     * @return ResultRespone
     */
    //(title = "修改工作书签信息表")
    @ApiOperation(value = "修改工作书签信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Views views) {
        return ResultRespone.success(viewsService.updateById(views));
    }

    /**
     * 通过id删除工作书签信息表
     *
     * @param id id
     * @return ResultRespone
     */
    //(title = "通过id删除工作书签信息表")
    @ApiOperation(value = "通过ID删除工作书签信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(viewsService.removeById(id));
    }

    /**
     * 通过id批量删除工作书签信息表
     *
     * @param ids id
     * @return ResultRespone
     */
    //(title = "通过id批量删除工作书签信息表")
    @ApiOperation(value = "通过ID批量删除工作书签信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(viewsService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
