package com.tofly.scada.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.scada.common.BaseController;
import com.tofly.scada.entity.TfScadaReportList;
import com.tofly.scada.service.TfScadaReportListService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 报警数据存储(TfScadaReportList)表控制层
 *
 * @author HaiQiu
 * @since 2022-09-21 16:21:20
 */
@Api(tags = "报警数据存储")
@RestController
@RequestMapping("tfScadaReportList")
public class TfScadaReportListController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private TfScadaReportListService tfScadaReportListService;

    /**
     * 分页查询数据
     *
     * @param page              分页对象
     * @param tfScadaReportList 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone pageList(Page page, TfScadaReportList tfScadaReportList) {
        return decide(tfScadaReportListService.pageList(page, tfScadaReportList));
    }

    /**
     * 查询所有数据
     *
     * @param tfScadaReportList 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone allList(TfScadaReportList tfScadaReportList) {
        return decide(tfScadaReportListService.allList(tfScadaReportList));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone get(@PathVariable Serializable id) {
        return decide(tfScadaReportListService.get(id));
    }

    /**
     * 新增数据
     *
     * @param tfScadaReportList 实体对象
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@RequestBody TfScadaReportList tfScadaReportList) {
        tfScadaReportListService.insert(tfScadaReportList);
        return decide("新增成功");
    }

    /**
     * 修改数据
     *
     * @param tfScadaReportList 实体对象
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@RequestBody TfScadaReportList tfScadaReportList) {
        tfScadaReportListService.edit(tfScadaReportList);
        return decide("修改成功");
    }

    /**
     * 批量修改数据状态
     *
     * @param idList 主键结合
     * @return 修改结果
     */
    @ApiOperation(value = "批量修改数据状态")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "idList", value = "数据ID数组集合"),
            @ApiImplicitParam(name = "status", value = "需要修改的数据ID状态值：0：未处理，1：已处理")
    })
    @PutMapping("/change/status")
    public ResultRespone change(@RequestParam("idList") List<Long> idList, @RequestParam String status) {
        tfScadaReportListService.change(idList,status);
        return decide("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        tfScadaReportListService.delete(idList);
        return decide("删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载", hidden = true)
    @GetMapping("/template")
    public void template() {
        tfScadaReportListService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入", hidden = true)
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        tfScadaReportListService.importData(file);
        return decide("导入成功");
    }


    /**
     * 数据导出
     *
     * @param page              分页
     * @param tfScadaReportList 查询模型
     * @param isAll             是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出", hidden = true)
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page<TfScadaReportList> page, TfScadaReportList tfScadaReportList, Boolean isAll) {
        tfScadaReportListService.export(page, tfScadaReportList, isAll);
    }
}

